package com.css.simulation.oauth.client.controller;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.SignUsernameParameter;
import api.common.pojo.param.SignSingleParameter;
import api.common.pojo.po.UserPO;
import api.common.pojo.vo.SimulationTokenVO;
import api.common.util.EncryptUtil;
import api.common.util.HttpUtil;
import api.common.util.JsonUtil;
import com.css.simulation.oauth.client.configuration.oauth.OauthParameter;
import com.css.simulation.oauth.client.mapper.UserMapper;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;

@Controller
@RequestMapping("/sign")
@Slf4j
public class SignController {

    @Resource
    private UserMapper userMapper;
    @Resource
    private CloseableHttpClient closeableHttpClient;
    @Resource
    private RequestConfig requestConfig;
    @Resource
    private OauthParameter oauthParameter;

    /**
     * http://localhost:6001/simulation/oauth/client/sign/hello
     * http://localhost:7001/simulation/oauth/client/sign/hello
     */
    @RequestMapping("/hello")
    @ResponseBody
    public String hello() {
        return "hello clent!";
    }


    /**
     * http://localhost:6001/simulation/oauth/client/entry?ticket=1001
     * http://localhost:7001/simulation/oauth/client/entry?ticket=1001
     */
    @RequestMapping("/entry")
    public String entry(@RequestParam("ticket") String ticket) {
        log.info(oauthParameter.toString());
        String zoogooyAuthorizeUrl = oauthParameter.getZoogooyAuthorizeUri() +
                "?appid=" + oauthParameter.getZoogooyAppid() +
                "&redirect_uri=" + oauthParameter.getZoogooyRedirectUri() +
                "&ticket=" + ticket +
                "&response_type=code" +
                "&scope=snsapi_userinfo";
        return "redirect:" + zoogooyAuthorizeUrl;
    }

    /**
     * 单点登录
     *
     * @return token 信息
     */
    @RequestMapping("/single")
    @SneakyThrows
    @ResponseBody
    public ResponseBodyVO<SimulationTokenVO> single(@RequestBody SignSingleParameter signSingleParameter) {
        String code = signSingleParameter.getCode();
        String ticket = signSingleParameter.getTicket();
        //1 根据统一凭条 code 获取统一平台 access_token

        String zoogooyTokenUrl = oauthParameter.getZoogooyTokenUri() + "?appid=" + oauthParameter.getZoogooyAppid() +
                "&secret=" + oauthParameter.getZoogooyAppSecret() +
                "&code=" + code +
                "&grant_type=authorization_code";


        String zoogooyTokenJson = HttpUtil.post(closeableHttpClient, requestConfig, zoogooyTokenUrl, null, null);
        ObjectMapper objectMapper1 = new ObjectMapper();
        JsonNode tokenRoot = objectMapper1.readTree(zoogooyTokenJson);
        String accessToken = tokenRoot.path("access_token").asText();
        String openid = tokenRoot.path("openid").asText();
        log.info("------- 统一平台令牌信息为：" + accessToken);

        //2 根据统一平台 access_token、openid、ticket 获取统一平台用户信息

        String zoogooyUserUrl = oauthParameter.getZoogooyUserUri() +
                "?access_token=" + accessToken +
                "&openid=" + openid +
                "&ticket=" + ticket;
        String zoogooyUserJson = HttpUtil.post(closeableHttpClient, requestConfig, zoogooyUserUrl, null, null);
        ObjectMapper objectMapper2 = new ObjectMapper();
        JsonNode userRoot = objectMapper2.readTree(zoogooyUserJson);

        String unionid = userRoot.path("unionid").asText();
        String nickname = userRoot.path("nickname").asText();
        log.info("------- 统一平台用户信息为：" + unionid);

        //3 使用 union_id 查询数据库，是否已在仿真平台存在该用户
        String username;
        String password;
        UserPO oldUser = userMapper.selectByIdIgnoreDelete(unionid);
        if (oldUser == null) {   //3-1 仿真平台不存在用户，直接创建新的
            password = EncryptUtil.getLowerMD5(oauthParameter.getSimulationDefaultPassword());
            UserPO newUser = new UserPO();
            newUser.setId(unionid);
            username = openid;  // 将 openid 作为登录名
            newUser.setUsername(username);
            newUser.setNickname(nickname);
            newUser.setPassword(password);
            userMapper.insert(newUser);
        } else {
            if ("1".equals(oldUser.getIsDeleted())) { //3-2 仿真平台存在删除状态用户，改为未删除
                userMapper.updateIsDeleted(unionid, "0");
            }
            //3-3 仿真平台用户存在未删除用户，放行
            username = oldUser.getUsername();
            password = oldUser.getPassword();
        }

        //4 根据仿真平台用户名密码颁发仿真平台 token，返回给前端
        String simulationTokenUrl = oauthParameter.getSimulationTokenUri() +
                "?grant_type=password" +
                "&client_id=" + oauthParameter.getSimulationClientId() +
                "&client_secret=" + oauthParameter.getSimulationClientSecret() +
                "&username=" + username +
                "&password=" + password;
        String simulationToken = HttpUtil.post(closeableHttpClient, requestConfig, simulationTokenUrl, null, null).toString();
        System.out.println("------- 仿真平台令牌信息为：" + simulationToken);
        SimulationTokenVO simulationTokenVO = JsonUtil.jsonToBean(simulationToken, SimulationTokenVO.class);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, simulationTokenVO);
    }


    /**
     * 用户名密码登录
     *
     * @return token 信息
     */
    @RequestMapping("/username")
    @SneakyThrows
    @ResponseBody
    public ResponseBodyVO<SimulationTokenVO> username(@RequestBody @Validated SignUsernameParameter signSingleParameter) {

        //1 使用 username 查询数据库，是否已在仿真平台存在该用户
        String username = signSingleParameter.getUsername();
        String password = signSingleParameter.getPassword();
        log.info("登录参数为："+signSingleParameter);
        UserPO oldUser = userMapper.selectByUsername(username);
        if (oldUser == null) {   //2-1 仿真平台不存在用户，直接创建新的
            return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE, "不存在该用户!");
        } else { //2-2 仿真平台存在用户判断密码
            if (!oldUser.getPassword().equals(password)) {
                return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE, "密码错误！");
            }
        }

        //3 根据仿真平台用户名密码颁发仿真平台 token，返回给前端
        String simulationTokenUrl = oauthParameter.getSimulationTokenUri() +
                "?grant_type=password" +
                "&client_id=" + oauthParameter.getSimulationClientId() +
                "&client_secret=" + oauthParameter.getSimulationClientSecret() +
                "&username=" + username +
                "&password=" + password;
        String simulationToken = HttpUtil.post(closeableHttpClient, requestConfig, simulationTokenUrl, null, null);
        System.out.println("------- 仿真平台令牌信息为：" + simulationToken);
        SimulationTokenVO simulationTokenVO = JsonUtil.jsonToBean(simulationToken, SimulationTokenVO.class);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, simulationTokenVO);
    }

}
